# base 2D grid world that draws a different color circle for each element type 
import entities
import occ_grid
import random
import math

CLOSE = 1
FAR = 2


#create one gatherer, one generator and resources 
def createEntities(grid):
    s1 = entities.Sub("yellow", 30, entities.Pos(random.uniform(0, 19), random.uniform(0, 14)), 0.25)
    a1 = entities.Atlantis(2, entities.Pos(random.uniform(10, 12), random.uniform(5, 10)))
    resources = spawnResources(grid, a1, 5)

    return s1, a1, resources 


# place entities in the grid
def placeEntities(grid, s1, a1, resources):
   grid.cells[int(s1.position.y)][int(s1.position.x)] = occ_grid.GATHERER
   grid.cells[int(a1.position.y)][int(a1.position.x)] = occ_grid.GENERATOR


# place resources with respect to the generator (a1) 
def spawnResources(grid, a1, Rlimit):
   resources = []   
   for i in xrange(0, Rlimit):
      s = 'sg'+repr(i)
      valid = 0
      while valid == 0:
        new_x = a1.position.x+random.uniform(-2, 2)
        new_y = a1.position.y+random.uniform(-2, 2)
        if grid.cells[int(new_y)][int(new_x)] == occ_grid.EMPTY:
         resources.append(entities.SeaGrass(s, entities.Pos(new_x, new_y)))
         grid.cells[int(new_y)][int(new_x)]  = occ_grid.RESOURCE
         valid = 1

   return resources

#reset the grid to try for a different target
def resetGrid(grid, startS, s1, a1, resources):
   grid.cells[int(startS.y)][int(startS.x)] = occ_grid.GATHERER
   for row in range(0, grid.height):
      for col in range(0, grid.width):
         if (grid.cells[row][col]  == occ_grid.GATHERERP or 
             grid.cells[row][col]  == occ_grid.GATHERER):
            grid.cells[row][col]  = occ_grid.EMPTY
   grid.cells[int(startS.y)][int(startS.x)] = occ_grid.GATHERER
   for n in resources:
      grid.cells[int(n.position.y)][int(n.position.x)] = occ_grid.RESOURCE
   grid.cells[int(a1.position.y)][int(a1.position.x)] = occ_grid.GENERATOR
   #reset actually gatherer
   s1.position.x = startS.x
   s1.position.y = startS.y
   

#grow a resource (only draws resource as R1, not real resource in world state)
def growResources(grid, resources, Rlocal):
   for r in resources:
      if grid.cells[int(r.position.y)][int(r.position.x)] == occ_grid.MARKED:
        for x in xrange(-Rlocal, Rlocal+1):
         for y in xrange(-Rlocal, Rlocal+1):
            new_x = r.position.x+x
            new_y = r.position.y+y 
            if (isValidPosition(grid, new_x, new_y) and 
                euclidDist(entities.Pos(new_x, new_y), r.position) <= Rlocal and
                grid.cells[int(new_y)][int(new_x)] == occ_grid.EMPTY):
               grid.cells[int(new_y)][int(new_x)] = occ_grid.RESOURCE1 

#helper function
def sign(n):
   if n >= 1:
      return 1
   elif n <= -1:
      return -1
   else:
      return 0


#distance function  note is squared - call on a Pos
def euclidDist(s1, d1):
    dir_x = d1.x - s1.x
    dir_y = d1.y - s1.y

    return math.sqrt(dir_x*dir_x + dir_y*dir_y) 
  
#travel function for sub's specific travel pattern
def determineNewGathererPosition(s1, sg1):
   dir_x = sign(int(sg1.position.x) - int(s1.position.x))
   dir_y = sign(int(sg1.position.y) - int(s1.position.y))

   if dir_x > 0:
      dir_y = 0

   new_x = s1.position.x + (dir_x * s1.speed)
   new_y = s1.position.y + (dir_y * s1.speed)
   return new_x, new_y


# determine if position is valid
def isValidPosition(grid, new_x, new_y):
   return (new_x >= 0 and new_x < grid.width and
      new_y >= 0 and new_y < grid.height)

#decide on the target resource either the closest or furthest
def determineTarget(s1, resources, mode):
   if mode==CLOSE:
      MinDist = 2000
      closest = resources[0]
      for n in resources:
         dist = euclidDist(s1.position, n.position)
         if dist <= MinDist:
            MinDist = dist
            closest = n 
      return closest
   else: 
      MaxDist = -1
      furthest = resources[0]
      for n in resources:
         dist = euclidDist(s1.position, n.position)
         if dist >= MaxDist:
            MaxDist = dist
            furthest = n
      return furthest

# update the positions of the entities
def updateEntities(grid, s1, a1, target): 
   
   new_x, new_y = determineNewGathererPosition(s1, target)

   if (isValidPosition(grid, new_x, new_y) ) :
      #set the old position to just a path value
      grid.cells[int(s1.position.y)][int(s1.position.x)] = occ_grid.GATHERERP
      s1.position.x = new_x
      s1.position.y = new_y
      #set the new position to the current gatherer position 
      grid.cells[int(new_y)][int(new_x)] = occ_grid.GATHERER

# update the extents of the resources 
def updateResources(grid, resources, Rlocal): 
   if Rlocal - int(Rlocal) < 0.005:
      growResources(grid, resources, int(Rlocal))
   if Rlocal < 3:
     Rlocal = Rlocal+.05
   return Rlocal
